#!/usr/bin/env python
# bibo-layers.py
# Created by TT
# This plug-in suggested by PixLab http://gimpchat.com/viewtopic.php?f=9&t=20604&p=284691&sid=b619396ab8f6bdcd5b5ae3e759753e28#p284691
# Altered code from bibo-text.py to handle linked layers instead of creating text layers.
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release
# Rel 2: Leave the visibility unchanged of linked layers, added "Change Opacity" and "Move Layers" toggles as suggested by PixLab http://gimpchat.com/viewtopic.php?p=284716#p284716

from gimpfu import *
import random


def ease_in_out(t):
    """Ease-in-out function: acceleration until halfway, then deceleration"""
    return 2*t*t if t < 0.5 else 1 - pow(-2*t + 2, 2) / 2
	
def python_bibo_layers(image, layer, cx,cy,minsize,maxsize,framesin,framesex,opacitychange,movelayers):

   pdb.gimp_image_undo_group_start(image)
   pdb.gimp_context_push()

   movelayers = int(movelayers)
   opacitychange=int(opacitychange)
   framesin = int(framesin)
   framesex = int(framesex)
   exhale = 0
   fontname = pdb.gimp_context_get_font()
   new_image = pdb.gimp_image_new(image.width,image.height,RGB)
   pdb.gimp_display_new(new_image)
   
   #get linked layers and reverse the order
   linkedlayers = []
   visible = []
   for l in image.layers:
      if pdb.gimp_item_get_linked(l) == TRUE:
         linkedlayers.append(l)
         visible.append(pdb.gimp_item_get_visible(l))
   linkedlayers.reverse()
   visible.reverse()

   #hide all linked layers so it doesn't affect our animation layers
   for l in linkedlayers:
      pdb.gimp_item_set_visible(l,FALSE)

   for i in range(0,len(linkedlayers)):
      linkedlayer = linkedlayers[i]
      if exhale == 0:
         frames = framesin
      else:
         frames = framesex   
      for j in range(0,frames):
         if exhale == 0:
            t = 1.0-float(j)/frames
         else:
            t = 0.0+float(j)/frames   
         size = minsize + (maxsize-minsize) * ease_in_out(t)
         opacity = 1.0 + (.2-1.0) * ease_in_out(t)
         #text_layer = pdb.gimp_text_fontname(image,None,0,0,word,-1,TRUE,size,0,fontname)
         new_layer = pdb.gimp_layer_new_from_drawable(linkedlayer,image)
         pdb.gimp_image_insert_layer(image,new_layer,None,0)
         pdb.gimp_item_set_visible(new_layer,TRUE)
         
         #auto crop to make it layer to content
         pdb.plug_in_autocrop_layer(image,new_layer)

         #scale it based on size/percentage
         pdb.gimp_layer_scale(new_layer,new_layer.width*size/100.0,new_layer.height*size/100.0,TRUE)
         if (movelayers == 1):
            pdb.gimp_layer_set_offsets(new_layer,cx-new_layer.width/2,cy-new_layer.height/2)
         if (opacitychange == 1):
            pdb.gimp_layer_set_opacity(new_layer,opacity * 100)
         visible_layer = pdb.gimp_layer_new_from_visible(image,new_image,'frame')
         pdb.gimp_image_insert_layer(new_image,visible_layer,None,0)
         pdb.gimp_image_remove_layer(image,new_layer)

      exhale = abs(exhale-1) #toggle between 0 and 1

   #set it back to whatever visibility it was   
   for i in range(0,len(linkedlayers)):
      l = linkedlayers[i]
      v = visible[i]
      pdb.gimp_item_set_visible(l,v)

   pdb.gimp_context_pop()
   pdb.gimp_image_undo_group_end(image)
   
   pdb.gimp_displays_flush()

    #return

register(
   "python_fu_bibo_layers",
   "Creates Bibo-layers from linked layers. Ver.2",
   "Creates Bibo-layers",
   "TT",
   "TT",
   "October 27, 2023",
   "<Image>/Filters/Animation/Animators/BIBO-layers...",
   "*",      # Create a new image, don't work on an existing one
   [
   #INPUT BEGINS
   (PF_INT, "cx", "Center-X:", 452),
   (PF_INT, "cy", "Center-Y:", 210),
   (PF_INT, "minsize", "Min Size %:", 10),
   (PF_INT, "maxsize", "Max Size %:", 100),
   (PF_INT, "framesin", "Frames Per Inhaling:", 15), #1.5 to 2 seconds
   (PF_INT, "framesout", "Frames Per Exhaling:", 20), #2 to 3 seconds.
   (PF_TOGGLE, "opacitychange",   "Opacity Change:", 1),
   (PF_TOGGLE, "movelayers",   "Move Layers To Given Center (x,y):", 1),
   #INPUT ENDS
   ],
   [],
   python_bibo_layers)

main()


